{
  "cells": [
    {
      "cell_type": "markdown",
      "id": "9b1f7a33",
      "metadata": {
        "id": "9b1f7a33"
      },
      "source": [
        "# Проверка ссылок на изображения в блокнотах и на сетевом диске"
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 1,
      "id": "46ef2099",
      "metadata": {
        "id": "46ef2099"
      },
      "outputs": [],
      "source": [
        "github_token = 'Input your access token here'\n",
        "dev_branches = ['dev-1.7', 'dev-1.8']"
      ]
    },
    {
      "cell_type": "code",
      "execution_count": 2,
      "id": "3b576929",
      "metadata": {
        "id": "3b576929",
        "colab": {
          "base_uri": "https://localhost:8080/"
        },
        "outputId": "98a115b5-f0f6-4b66-be03-d1a780d695ef"
      },
      "outputs": [
        {
          "output_type": "stream",
          "name": "stdout",
          "text": [
            "Get host links...\n",
            "Host links received!\n",
            "Get secret links:\n",
            "\tGet branch dev-1.7\n",
            "\tGet branch dev-1.8\n",
            "All links received!\n",
            "Get lectures links:\n",
            "\tGet branch dev-1.7\n",
            "\tGet branch dev-1.8\n",
            "All links received!\n"
          ]
        }
      ],
      "source": [
        "from bs4 import BeautifulSoup\n",
        "import requests\n",
        "import base64\n",
        "import json\n",
        "import os\n",
        "import re\n",
        "\n",
        "class Url:\n",
        "    # this class exists solely to make url comparisons ignore whether the url uses http or https\n",
        "    def __init__(self, string):\n",
        "        self.protocol, self.address = string.split(\"://\")\n",
        "    def __eq__(self, other):\n",
        "        return isinstance(other, Url) and self.address == other.address\n",
        "    def __str__(self):\n",
        "        return self.protocol + '://' + self.address\n",
        "    def __hash__(self):\n",
        "        return hash(self.address)\n",
        "    def __lt__(self, other):\n",
        "        return isinstance(other, Url) and self.address < other.address\n",
        "def get_links_from_dir(url, blacklist=()):\n",
        "    # recursive fuction that walks a directory tree on a server and collects all the images\n",
        "    # a blacklisted directory is not checked for images\n",
        "    for s in blacklist:\n",
        "        if s in url:\n",
        "            return []\n",
        "    html_text = requests.get(url).text\n",
        "    img_pattern = re.compile(r\"https?://.*\\.(png|jpe?g|gif|svg)\\\\?\")\n",
        "    image_links = BeautifulSoup(html_text, 'lxml').find_all(href=True)[1:]\n",
        "    lecture_links = []\n",
        "    for link in image_links:\n",
        "        href = link.get('href')\n",
        "        if href[-1] == '/':\n",
        "            lecture_links += get_links_from_dir(url + href)\n",
        "        else:\n",
        "            if not img_pattern.search(url + href):\n",
        "                continue\n",
        "            lecture_links.append(Url(url + href))\n",
        "    return lecture_links\n",
        "def get_host_links(lec_num, blacklist=()):\n",
        "    # dict for links\n",
        "    host_links = {}\n",
        "    print('Get host links...')\n",
        "    # for each lecture\n",
        "    for lecture in lec_num:\n",
        "        # images on the server are generally stored in three locations:\n",
        "        # - the EduNet-content folder\n",
        "        # - the EduNet-web_dependencies folder\n",
        "        # - thr src folder (deprecated)\n",
        "        lecture_links = []\n",
        "        # recursively searches the Edunet-content folder for this lecture\n",
        "        url = f'https://edunet.kea.su/repo/EduNet-content/{lecture}/'\n",
        "        lecture_links += get_links_from_dir(url, blacklist)\n",
        "        # recursively searches the Edunet-web_dependencies folder for this lecture\n",
        "        url = f'https://edunet.kea.su/repo/EduNet-web_dependencies/{lecture}/'\n",
        "        lecture_links += get_links_from_dir(url, blacklist)  # recursively searches EduNet-web_dependencies\n",
        "        # the directories in src are called by the full lecture title rather than just the number, so a match is needed\n",
        "        # to select the right one to search\n",
        "        url = f'https://edunet.kea.su/repo/src/'\n",
        "        lecture_name_pattern = re.compile(r\"(L\\d+)(\\w+)\")\n",
        "        html_text = requests.get(url).text\n",
        "        dir_links = BeautifulSoup(html_text, 'lxml').find_all(href=True)\n",
        "        for link in dir_links:\n",
        "            match = lecture_name_pattern.match(link.get('href'))\n",
        "            if match and match[1] == lecture:\n",
        "                # recursively searches the discovered folder\n",
        "                lecture_links += get_links_from_dir(f'https://edunet.kea.su/repo/src/{match[0]}/img_license/', blacklist)\n",
        "                lecture_links += get_links_from_dir(f'https://edunet.kea.su/repo/src/{match[0]}/img/', blacklist)\n",
        "        # add all links in dict, like dict{'L01':\"all L01 links\", 'L02': all L02 links ..... etc}\n",
        "        host_links[lecture] = set(lecture_links) - ignore_set\n",
        "    print('Host links received!')\n",
        "    return host_links\n",
        "def match_patterns(string, patterns):\n",
        "    match = False\n",
        "    for p in patterns:\n",
        "        match = p.search(string)\n",
        "        if match:\n",
        "            break\n",
        "    return match\n",
        "def strip_protocol(url: str):\n",
        "    return url.lstrip(\"https\")\n",
        "def get_lectures_links(dev_branches, github_token, repo_url):\n",
        "    # iterates over the branches of a github repository, and searches lecture files within for links to images\n",
        "    img_pattern = re.compile(r\"<img(?:\\s[^<>\\\"\\']*)?\\ssrc\\s*=\\s*\\\"([^<>\\\"\\']*)\\\".*?>\")  # pattern for image markdown in notebook\n",
        "    # pattern for a url to an image stored on the server\n",
        "    path_pattern = re.compile(\n",
        "        r\"https?://edunet\\.kea\\.su/repo/(EduNet-(?:(?:content)|(?:web_dependencies))/(L\\d+)/\\S+?\\.(png|jpe?g|gif|svg))\")\n",
        "    non_disk_pattern = re.compile(r\"https?://[^\\\"\\']*?\\.(png|jpe?g|gif|svg)\\\\?\")  # a general image url pattern\n",
        "    url_patterns = [path_pattern, non_disk_pattern]\n",
        "    var_patten = re.compile(r'(?:' + path_pattern.pattern + r\"|\" + non_disk_pattern.pattern + r')')\n",
        "    headers = {}\n",
        "    if github_token:\n",
        "        headers['Authorization'] = f\"token {github_token}\"\n",
        "    res_links = {}\n",
        "    # get the lecture names for the default branch\n",
        "    url = repo_url + f'/contents/out?ref={dev_branches[0]}'\n",
        "    r = requests.get(url, headers=headers)\n",
        "    r.raise_for_status()\n",
        "    lec_data = r.json()\n",
        "    for branch in dev_branches:\n",
        "        print(f'\\tGet branch {branch}')\n",
        "        lecture_names_tmp = [x['name'] for x in lec_data[2:17]]\n",
        "        lecture_names = []\n",
        "        for lecture in lecture_names_tmp:\n",
        "            if lecture.endswith(\".ipynb\"):\n",
        "                lecture_names.append(lecture)\n",
        "        for lecture in lecture_names:\n",
        "            # get the contents of the lecture folder on this branch\n",
        "            url = repo_url + f\"/contents/out/{lecture}?ref={branch}\"\n",
        "            r = requests.get(url, headers=headers)\n",
        "            r.raise_for_status()\n",
        "            data = r.json()\n",
        "            if 'content' not in data:  # shorthand for a filename being a directory\n",
        "                for d in data:\n",
        "                    lecture_names.append(lecture + '/' + d['name'])  # breadth-first search over subdirectories\n",
        "                continue\n",
        "            if 'ipynb' not in url:  # the only non-directory files we are searching for are jupyter notebooks\n",
        "                continue\n",
        "            file_content = data['content']\n",
        "            file_content_encoding = data.get('encoding')\n",
        "            if file_content_encoding == 'base64':\n",
        "                file_content = base64.b64decode(file_content).decode()\n",
        "            file_content = json.loads(file_content)\n",
        "            links = []\n",
        "            for cell in file_content[\"cells\"]:\n",
        "                lines = [s.strip(\" \\r\\t\\n\\\"\\'\") for s in cell[\"source\"]]\n",
        "                text = \"\".join(lines)\n",
        "                if cell[\"cell_type\"] == 'code':  # search source code files for urls used as variables\n",
        "                    for match in var_patten.finditer(text):\n",
        "                        if match:\n",
        "                            links.append(Url(match.group(0)))\n",
        "                for match in img_pattern.finditer(\"\".join(lines)):  # search for embedded images in every cell\n",
        "                    if not match:\n",
        "                        continue\n",
        "                    link = match.group(1)\n",
        "                    match = match_patterns(link, url_patterns)\n",
        "                    if match:\n",
        "                        links.append(Url(link))\n",
        "            lcode = str(lecture[:3])\n",
        "            if lcode in res_links:\n",
        "                res_links[lcode] = res_links[lcode].union(set(links))\n",
        "            else:\n",
        "                res_links[lcode] = set(links)\n",
        "    print('All links received!')\n",
        "    return res_links\n",
        "\n",
        "if __name__ == '__main__':\n",
        "    ignore_list = ['https://edunet.kea.su/repo/EduNet-web_dependencies/L02/student_scores.csv',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L03/heart.csv',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L04/lfwcrop_grey.zip',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L04/rnaseq_data.tab.txt',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L04/titanic.csv',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L04/scRNAseq_CITEseq.txt',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L04/fetal_health.csv',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L05/lc_cifar10_weights.txt',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L08/airline-passengers.csv',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L09/imagenet_class_index.json',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L09/imagen.zip',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L09/imagenet1000_clsidx_to_labels.txt',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L10/imagenet_class_index.json',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L10/boston_dataset.csv',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L11/OSCD.zip',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L11/OSCD2.zip',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L11/audio_example.wav',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L11/GTdb_crop.zip',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L11/for_transforms.Compose.zip',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L11/EuroSAT.zip',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L11/small_face_dataset.zip',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L12/cheetah_video.mp4',\n",
        "                   'https://edunet.kea.su/repo/EduNet-web_dependencies/L13/weights_stylegan.pt']\n",
        "    \n",
        "    ignore_set = set([Url(s) for s in ignore_list])\n",
        "\n",
        "    blacklist = ['https://edunet.kea.su/repo/src']\n",
        "\n",
        "    lec_num = ['L01', 'L02', 'L03', 'L04', 'L05', 'L06', 'L07', 'L08', 'L09', 'L10', 'L11', 'L12', 'L13', 'L14', 'L15']\n",
        "    host_links = get_host_links(lec_num, blacklist)\n",
        "    print(\"Get secret links:\")\n",
        "    secret_links = get_lectures_links(dev_branches, github_token, 'https://api.github.com/repos/EPC-MSU/EduNet-secret')\n",
        "\n",
        "    print(\"Get lectures links:\")\n",
        "    lectures_links = get_lectures_links(dev_branches, github_token, 'https://api.github.com/repos/EPC-MSU/EduNet-lectures')\n",
        "    \n",
        "    # make sure that all the dictionaries have fields for every lecture\n",
        "    for key in lec_num:\n",
        "        if key not in lectures_links:\n",
        "            lectures_links[key] = set()\n",
        "        if key not in secret_links:\n",
        "            secret_links[key] = set()\n",
        "\n",
        "    # generate a union of all host links\n",
        "    host_links['all'] = set()\n",
        "    for i in lec_num:\n",
        "        host_links['all'] = host_links['all'].union(host_links[i])\n",
        "    with open(\"links_info.txt\", \"w+\") as f:\n",
        "        for i in lec_num:\n",
        "            print(f'* * * Лекция {i} * * * \\n', file=f)\n",
        "            print(f'Есть на сетевом нет в лекции:\\n', file=f)\n",
        "            lectures_links[i] = lectures_links[i]\n",
        "            if i not in host_links:\n",
        "                host_links[i] = set()\n",
        "            if i not in lectures_links:\n",
        "                lectures_links[i] = set()\n",
        "            for link in sorted(list(host_links[i] - lectures_links[i])):\n",
        "                print(f'\\t{link}\\n', file=f)\n",
        "            print(f'\\nЕсть в лекции нет на сетевом: \\n', file=f)\n",
        "            if len(list(lectures_links[i] - host_links['all'])) == 0:\n",
        "                print('Все ссылки есть на сетевом!\\n', file=f)\n",
        "            else:\n",
        "                for link in sorted(list(lectures_links[i] - host_links['all'])):\n",
        "                    print(f'\\t{link}\\n', file=f)\n",
        "\n",
        "            # prints warnings when a lecture links to another lecture's folder\n",
        "            # for j in lec_num:\n",
        "            #     if i == j:\n",
        "            #         continue\n",
        "            #     links = sorted(list(lectures_links[i].intersection(host_links[j])))\n",
        "            #     if len(links) != 0:\n",
        "            #         print(f'\\nНет на сетевом в папке этой лекции, но есть в папке {j}: \\n', file=f)\n",
        "            #     for link in links:\n",
        "            #         print(f'\\t{link}\\n', file=f)\n",
        "            print(f'\\nЕсть в заданиях нет на сетевом: \\n', file=f)\n",
        "            if len(list(secret_links[i] - host_links[i])) == 0:\n",
        "                print('Все ссылки есть на сетевом!\\n', file=f)\n",
        "            else:\n",
        "                for link in sorted(list(secret_links[i] - host_links[i])):\n",
        "                    print(f'\\t{link}\\n', file=f)"
      ]
    }
  ],
  "metadata": {
    "colab": {
      "provenance": []
    },
    "language_info": {
      "name": "python"
    },
    "kernelspec": {
      "name": "python3",
      "display_name": "Python 3"
    },
    "gpuClass": "standard"
  },
  "nbformat": 4,
  "nbformat_minor": 5
}